## library required packages
library(sampling)
library(foreach)
library(plyr)
library(dismo)
library(gbm)
library(matrixStats)
library(tidyverse)


## load the required data, with the first 825 rows representing the virus data and the remaining rows 
## representing the data for explanatory factors (excluding those grid cells having the same year 
## and coordinate as the discovery year and coordinate for any viruses).
load('all_dat.RData')
## Notes for the data set
## Column 1: integer, ranges 1~224. virusID = 1-223, IDs for the 223 viruses, some viruses have repeated IDs because their discovery locations are polygons; virusID = 224, 'virus free' years and grid cells.
## Column 2: binary(0,1). frequency = 1, virus data; frequency = 0, data for 'virus free' years and grid cells.
## Column 3 and 4: lon signifies longitude (numeric, -179.5~179.5), lat signifies latitude (numeric, -55.5~82.5). This column shows coordinates of discovery locations or grid cells across the globe. 
## Column 5: integer, ranges 1900~2015. This column shows discovery decade (with the true discovery year rounded off) or every decadal year in 1900~2015 (data for 2020 is not available, so data for 2015 is used instead).
## Column 6~38: the 33 explanatory factors. The full name for each explanatory factor can be found in the script shown below (lines 90-122)


## using foreach, sampling data n times and create a list, sampled_dat_n. n=1000 for our current study.
## each sample includes 223 grid cells with virus discovery and 446 with no virus discovery were included 
virus_dat <- subset(all_dat, frequency==1) 
con_dat <- subset(all_dat, frequency==0) 
n <- 1000
sampled_dat_n <- foreach(i = 1:n) %do% {
  x <- strata(virus_dat, "virusID", size = c(rep(1,223)), method = "srswor")
  dat_i <- getdata(all_dat, x)
  dat_i <- dat_i[,1:37]
  freq <- with(dat_i, table(dat_i$lon,dat_i$lat,dat_i$year))
  freq <- as.data.frame(freq)
  names(freq) <- c("lon","lat","year","Freq")
  freq[,1] <- as.numeric(as.character(freq[,1]))
  freq[,2] <- as.numeric(as.character(freq[,2]))
  freq[,3] <- as.numeric(as.character(freq[,3]))
  freq[,4] <- as.numeric(as.character(freq[,4]))
  freq <- freq[freq$Freq > 1,]
  dat_i <- left_join(dat_i, freq, by= c("lon","lat","year"))
  dat_i <- within(dat_i,{
    frequency[!is.na(Freq)] <- Freq[!is.na(Freq)]
  })
  dat_i <- unique(dat_i[,-38])
  dat_i <- arrange(dat_i,year)
  y <- as.data.frame(table(dat_i$year))
  y$Var1 <- as.numeric(as.character(y$Var1))
  y1 <- c(1900,1910,1920,1930,1940,1950,1960,1970,1980,1990,2000,2010,2015)
  y2 <- y1[-which(y1 %in% y$Var1)]
  con_dat1 <- con_dat[-which(con_dat$year %in% y2),]
  con_dat1 <- arrange(con_dat1,year)
  x <- strata(con_dat1, "year", size = y$Freq*2, method = "srswor")
  dat_i_con <- getdata(con_dat1, x)
  dat_i_con <- dat_i_con[,c(2:4,38,5:37)]
  dat_i_con$frequency <- as.numeric(dat_i_con$frequency)
  dat_i <- rbind(dat_i,dat_i_con)
  return(dat_i)
}

rm(dat_i,freq,x,y,y1,y2,i,n,con_dat1,dat_i_con)


## generate a vector with all explanatory factor names
predictor_names <- c("tmp","tmp_change","tmx","tmn","dtr","pre","pre_change","premx","premn","wet","gcrop","gcrop_change", "gpast",
                     "gpast_change","gurbn", "gurbn_change","gpri","gpri_change","gsecd","gsecd_change","gflcu","gflpu","gflvu","gflsu",
                     "popcount","popcount_change","GDP", "GDP_change","vegetation","mamms" ,"livestock",
                     "unifre","latitude")

# using foreach, perform gbm.step for each sampled_dat_n[[i]]
bsm <- foreach(i = 1:length(sampled_dat_n), .verbose = TRUE) %dopar% {
  sampled_data_i <- as.data.frame(sampled_dat_n[[i]])
  x_ind <- which(colnames(sampled_data_i) %in% predictor_names)
  y_ind <- which(colnames(sampled_data_i) %in% "frequency")
  model <- gbm.step(data = sampled_data_i, 
                    gbm.x = x_ind, 
                    gbm.y = y_ind, 
                    family = "poisson", 
                    tree.complexity = 5,
                    learning.rate = 0.003,
                    bag.fraction = 0.5,
                    plot.main = FALSE, verbose = TRUE)
  return(model)
}


bsm <- bsm[!sapply(bsm, is.null)]# some models may be invalid

# draw Relative Contribution Plot for the n=1000 replicate models ####
names <- c("tmp" = "Mean temperature",
           "tmp_change" = "Temperature change",
           "tmn" ='Minimum temperature',
           "tmx" = 'Maximum temperature',
           "dtr"='Diurnal temperature range',
           "pre"='Total precipitation', 
           "premx"='Maximum precipitation',
           "premn"='Minimum precipitation', 
           "pre_change"='Precipitation change',
           "wet"='Annual rainy days',
           "popcount"='Population count',
           "popcount_change"='Population growth',
           "GDP"='GDP',
           "GDP_change"='GDP growth',
           "gcrop"='Cropland',
           "gcrop_change"='Growth of cropland area',
           "gpast"='Pasture',
           "gpast_change"='Growth of pasture area',
           "gurbn"='Urbanized land',
           "gurbn_change"='Growth of urbanized land area',
           "gpri"='Primary land',
           "gpri_change"='Growth of primary land area',
           "gsecd"='Secondary land',
           "gsecd_change"='Growth of secondary land area',
           "gflcu"='Urbanization of cropland',
           "gflpu"='Urbanization of pasture',
           "gflvu"='Urbanization of primary land',
           "gflsu"='Urbanization of secondary land',
           "vegetation"='Vegetation',
           "mamms"='Mammal species richness',
           "livestock"='Livestock headcount',
           "unifre"='University count',
           "latitude"='Latitude')
  

groups <- list("Climatic" = "tmp",
               "Climatic" = "tmp_change",
               "Climatic" = "tmx",
               "Climatic" = "tmn",
               "Climatic" = "dtr",
               "Climatic" = "pre",
               "Climatic" = "pre_change",
               "Climatic" = "premx",
               "Climatic" = "premn",
               "Climatic" = "wet",
               'Climatic'= "latitude",
               "Socio-economic" = "popcount",
               "Socio-economic" = "popcount_change",
               "Socio-economic" = "GDP",
               "Socio-economic" = "GDP_change",
               "Socio-economic" = "unifre",
               "Land use" = "gcrop",
               "Land use" = "gcrop_change",
               "Land use" = "gpast",
               "Land use" = "gpast_change",
               "Land use" = "gurbn",
               "Land use" = "gurbn_change",
               "Land use" = "gpri",
               "Land use" = "gpri_change",
               "Land use" = "gsecd",
               "Land use" = "gsecd_change",
               "Land use" = "gflcu",
               "Land use" = "gflpu",
               "Land use" = "gflvu",
               "Land use" = "gflsu",
               "Land use" = "vegetation",
               "Biodiversity" = "mamms",
               "Biodiversity" = "livestock")
  

bsm_scatter <- ldply(1:length(bsm), .fun = function(i) {
  y <- summary(bsm[[i]], plotit = FALSE)
  y$i <- i
  return(y)
  }, .parallel = TRUE)
  
bsm_rel_inf <- ddply(bsm_scatter, c("var"), summarize, rel.inf.med = median(rel.inf))
bsm_rel_inf$var <- as.character(bsm_rel_inf$var)
bsm_rel_inf <- bsm_rel_inf[order(bsm_rel_inf$rel.inf.med), ]
bsm_scatter$var <- factor(bsm_scatter$var, levels = bsm_rel_inf$var)
bsm_scatter$Group <- factor(bsm_scatter$var)
levels(bsm_scatter$Group) <- groups


relative_contribution <- ggplot(bsm_scatter, aes(x = var, y = rel.inf, fill = Group)) + 
  geom_boxplot()+
  coord_flip() + 
  scale_x_discrete(labels = names) + 
  labs(y = "Relative Contribution (%)", x = "Explanatory factor") +
  theme_bw(base_size = 11) +
  theme(axis.text = element_text(size = 7,face= "bold", colour = "black")) +
  theme(axis.title= element_text(size=10, face= "bold")) +
  theme(legend.title = element_text(size=9, face="bold")) +
  theme(legend.text = element_text(size = 7,face= "bold")) +
  theme(legend.position = c(0.9, 0.2))

relative_contribution

## Export values of relative_contribution for each explanatory factor, relative_contribution_median
relative_contribution_median <-
  bsm_scatter %>% 
  group_by(var)%>% 
  dplyr::summarise(Median=median(rel.inf))%>% 
  arrange(-Median)

## Export deviance of the bootstrap model, dev3
dev <- foreach(i = 1:length(bsm), .verbose = FALSE, .combine = rbind) %do% {
 value <- data.frame(1-(bsm[[i]]$cv.statistics$deviance.mean/bsm[[i]]$self.statistics$mean.null))
}
names(dev) <- 'dev'
dev2 <- as.matrix(dev)
dev3 <- data.frame(colQuantiles(dev2, probs = c(0.025, 0.5, 0.975)))

## Export number of trees, nt_mean
nt <- foreach(i = 1:length(bsm), .verbose = FALSE, .combine = rbind) %do% {
  value <- data.frame(bsm[[i]]$n.trees)
}
names(nt) <- 'nt'
nt_mean <- mean(nt$nt)

## Plot partial_dependence_plots ####
to_plot <- bsm[[1]]$gbm.call$predictor.names
partial_dependence_raw <- list()
for(v in 1:length(to_plot)) {
  cat(paste0("Working on ", to_plot[v], "...\n"))
  pdvar <- foreach(i = 1:length(bsm), .verbose = FALSE, .combine = rbind) %do% {
    p <- plot.gbm(bsm[[i]], i.var = to_plot[v], return.grid = TRUE)
    p <- data.frame(p, i)
  }
  pdvar$name <- to_plot[v]
  names(pdvar) <- c("x", "y", "i", "name")
  pdvar[,2] <- scale(pdvar[,2],scale = FALSE)
  partial_dependence_raw[[v]] <- pdvar
}
pdq <- list()
for(v in 1:length(partial_dependence_raw)) {
  var <- partial_dependence_raw[[v]]
  name <- var$name[1]
  cat(paste0("Working on ", name, "...\n"))
  s <- seq(min(var$x), max(var$x), length.out = 101)
  a <- array(c(s[1:length(s)-1], s[2:length(s)]), dim = c(length(s) - 1, 2))
  var2 <- adply(a, 1, function(i) {
    y <- var[var$x >= i[1] & var$x < i[2], "y"]
    x <- mean(c(i[1], i[2]))
    q <- (quantile(y, c(0.025, 0.5, 0.975), names = FALSE, na.rm = TRUE))
    df <- data.frame(t(c(x, q)))
    names(df) <- c("x", "q05", "q50", "q95")
    df$name <- name
    return(df)
  })
  pdq[[v]] <- var2[2:length(var2)]
}
pdq <- do.call(rbind, pdq)
pdq$name <- factor(pdq$name, levels = relative_contribution_median$var) # This is for variable ordering on the plot


names <- c("tmp" = "Mean temperature (\u00B0C)",
           "tmp_change" = "Temperature change (\u00B0C)",
           "tmn" ='Minimum temperature (\u00B0C)',
           "tmx" = 'Maximum temperature (\u00B0C)',
           "dtr"='Diurnal temperature range \n(\u00B0C)',
           "pre"='Total precipitation (/100, mm)', 
           "premx"='Maximum precipitation \n(/100, mm)',
           "premn"='Minimum precipitation\n(/100, mm)', 
           "pre_change"='Precipitation change\n(/100, mm)',
           "wet"='Annual rainy days (counts)',
           "popcount"='Population count\n(/100, counts)',
           "popcount_change"='Population growth\n(/100, counts)',
           "GDP"='GDP\n(PPP, billion US$2005/yr)',
           "GDP_change"='GDP growth\n(PPP, billion US$2005/yr)',
           "gcrop"='Cropland (*100%)',
           "gcrop_change"='Growth of cropland area\n(*100%)',
           "gpast"='Pasture (*100%)',
           "gpast_change"='Growth of pasture area\n(*100%)',
           "gurbn"='Urbanized land (*100%)',
           "gurbn_change"='Growth of urbanized\nland area (*100%)',
           "gpri"='Primary land (*100%)',
           "gpri_change"='Growth of primary\nland area (*100%)',
           "gsecd"='Secondary land (*100%)',
           "gsecd_change"='Growth of secondary\nland area (*100%)',
           "gflcu"='Urbanization of cropland\n(*100%)',
           "gflpu"='Urbanization of pasture\n(*100%)',
           "gflvu"='Urbanization of\nprimary land (*100%)',
           "gflsu"='Urbanization of\nsecondary land (*100%)',
           "vegetation"='Vegetation (%)',
           "mamms"='Mammal species richness\n(counts)',
           "livestock"='Livestock headcount\n(/100, counts)',
           "unifre"='University count (counts)',
           "latitude"='Latitude (\u00B0C)')

groups <- list("Climatic" = "tmp",
               "Climatic" = "tmp_change",
               "Climatic" = "tmx",
               "Climatic" = "tmn",
               "Climatic" = "dtr",
               "Climatic" = "pre",
               "Climatic" = "pre_change",
               "Climatic" = "premx",
               "Climatic" = "premn",
               "Climatic" = "wet",
               'Climatic'= "latitude",
               "Socio-economic" = "popcount",
               "Socio-economic" = "popcount_change",
               "Socio-economic" = "GDP",
               "Socio-economic" = "GDP_change",
               "Socio-economic" = "unifre",
               "Land use" = "gcrop",
               "Land use" = "gcrop_change",
               "Land use" = "gpast",
               "Land use" = "gpast_change",
               "Land use" = "gurbn",
               "Land use" = "gurbn_change",
               "Land use" = "gpri",
               "Land use" = "gpri_change",
               "Land use" = "gsecd",
               "Land use" = "gsecd_change",
               "Land use" = "gflcu",
               "Land use" = "gflpu",
               "Land use" = "gflvu",
               "Land use" = "gflsu",
               "Land use" = "vegetation",
               "Biodiversity" = "mamms",
               "Biodiversity" = "livestock")


pdq$Group <- factor(pdq$name)
levels(pdq$Group) <- groups
pdq$name <- revalue(pdq$name, replace = names)

partial_dependence_plot <-  ggplot(pdq, aes(x = x)) +
  facet_wrap(~ name, scales = "free", ncol =5) +
  ylim(-1.0,  0.7 ) + # Fix y axes for rigor :)
  geom_ribbon(aes(ymin = q05, ymax = q95, fill = Group), alpha = 0.75) +
  geom_line(aes(y = q50)) + 
  theme_bw(base_size = 11, base_family = "") +
  labs(x = "Value of explanatory factor", y = "Marginal effect") +
  theme(axis.text = element_text(face = "bold", size = 7, colour = "black")) + 
  theme(axis.title= element_text(size=13, face= "bold")) +
  theme(panel.background = element_rect(fill = "white", colour = "grey50")) +
  theme(legend.title = element_text(size=10, face="bold")) +
  theme(legend.text = element_text(size = 9,face="bold")) +
  theme(legend.position = c(0.8, 0.04))+
  theme(strip.background = element_rect(fill="white"))+
  theme(strip.text = element_text(size = 10,face = "bold"))

partial_dependence_plot

## predicted probability in 2010-2019 #####
## get data of explanatory factors for all grid cells across the globe in 2015
covariates <- unique(all_dat[,-c(1:2)])
covariates_2015 <- covariates[which(covariates$year==2015),]

## get the predicted virus count in each grid cell based on the fitted model (bsm) and explanatory factors of 2015
predict_multibrt <- function(multibrt, newdata, type = "response", value = "mean") {
  library(matrixStats)
  library(purrr)
  prediction_matrix <- multibrt %>%
    map(~ predict(.x, newdata, n.trees = .x$n.trees, type = type)) %>%
    flatten_dbl() %>%
    matrix(nrow = nrow(newdata))
  if (value == "mean") {
    return(rowMeans(prediction_matrix))
  } else if (value == "sd") {
    return(rowSds(prediction_matrix))
  } else if (value == "matrix") {
    return(prediction_matrix)
  }
}

covariates_2015$bsm_response <- predict_multibrt(bsm, covariates_2015, type = "response", value = "mean")
predictions <- dplyr::select(covariates_2015, lon, lat,bsm_response)

## convert to probability based on the equation of Poisson probability distribution
predictions$prob <- 1-(1/(2.171828^predictions$bsm_response))
predictions <- predictions[,c(1,2,4)]

## save plot as .tif and input to ArcGIS to get it visualised
predictions_tif <- rasterFromXYZ(predictions)             
#plot(predictions_tif)
writeRaster(predictions_tif, "./pred", format = "GTiff")
